# Load packages and functions-----------------------------------------------

#Installations
#pkgs <- c("lme4","dplyr","bblme","ggplot2","scales","mgcv","reshape2",
#          "MASS","multcomp","rcompanion","car","glmmTMB","emmeans")
#install.packages(pkgs)

#Load packages
library(dplyr)
library(reshape2)
library(ggplot2)
library(scales)
library(mgcv)
library(bbmle)
library(MASS)
library(multcomp)
library(rcompanion)
library(car)
library(glmmTMB)
library(lme4)
library(emmeans)

#Skip comp function for successive differences contrasts in emmeans
#From: https://cran.r-project.org/web/packages/emmeans/vignettes/comparisons.html
skip_comp.emmc <- function(levels, skip = 1, reverse = FALSE) {
  if((k <- length(levels)) < skip + 1)
    stop("Need at least ", skip + 1, " levels")
  coef <- data.frame()
  coef <- as.data.frame(lapply(seq_len(k - skip - 1), function(i) {
    sgn <- ifelse(reverse, -1, 1)
    sgn * c(rep(0, i - 1), 1, rep(0, skip), -1, rep(0, k - i - skip - 1))
  }))
  names(coef) <- sapply(coef, function(x)
    paste(which(x == 1), "-", which(x == -1)))
  attr(coef, "adjust") = "fdr"   # default adjustment method
  coef
}

#Rescale coefficients function from:
#https://stackoverflow.com/questions/35209579/unscale-predictor-coefficients-lmer-model-fit-with-an-unscaled-response?rq=1
get_real <- function(coef, scaled_covariate){
  
  # collect mean and standard deviation from scaled covariate
  mean_sd <- unlist(attributes(scaled_covariate)[-1])
  
  # reverse the z-transformation
  answer <- (coef * mean_sd[2]) + mean_sd[1]
  
  # this value will have a name, remove it
  names(answer) <- NULL
  
  # return unscaled coef
  return(answer)
}

# 1) Water quality and tidal emersion data -----------------------------------

#Load and clean water quality data
wq <- read.csv('wq_data.csv') %>% 
  sapply(function(x) {x <- gsub("Dry",NA,x)}) %>% #Replace "Dry" with "NA"
  as.data.frame() %>% 
  mutate(temp = temp %>% as.character() %>% as.numeric(),
         DO = DO %>% as.character() %>% as.numeric(),
         pH = pH %>% as.character() %>% as.numeric(),
         salinity = salinity %>% as.character() %>% as.numeric(), #Factors to numeric
         date = date %>% as.character() %>% as.Date(format = "%d-%b") %>% 
           format("%j") %>% as.numeric(),
         elevation = factor(elevation, levels = c("Ocean", "Low", "Medium", "High"),
                            labels = c("Subtidal", "Low", "Middle", "High")))

#Summarize wq measures (Table 2.1; rows 1-3)
wq %>% melt(id.vars = c("date", "location", "elevation")) %>% 
  filter(!is.na(value)) %>% 
  group_by(elevation, variable) %>% 
  summarise(mean = mean(value), sd = sd(value), min = min(value),
            max = max(value), n = n())

#Load emersion data
emersion <- read.csv('exposure_durations.csv') %>% 
  mutate(elevation = factor(elevation, levels = c("low", "mid", "high"),
                            labels = c("Low", "Middle", "High")))

#Mean +/- sd duration emersed (h) and number of days emersed (Table 2.1; row 4)
emersion %>% group_by(elevation) %>% 
  summarise(Mean_Dur = mean(duration), SD_Dur = sd(duration),
            N_days = n())

# 2) Temperature data --------------------------------------------------------

#Load temperature data collected from all loggers in 2018
temp <- read.csv("temp_data_full.csv") %>% 
  mutate(date = date %>% as.character() %>% as.Date(format = "%d-%b-%y")) %>% 
  mutate(cal_date = date, date = date %>% format ("%j") %>% as.numeric()) %>% 
  filter(date > 133) %>% #remove data from early May prior to formal study start
  mutate(day = date - min(date) + 1,
         elevation = factor(elevation_num, levels = c(1, 2, 3, 4),
                            labels = c("Subtidal","Low","Middle","High")))

#Date ranges by elevation
(date.ranges <- left_join(summarise(group_by(temp, elevation), min_date = min(date)),
                         summarise(group_by(temp, elevation), max_date = max(date))) %>%
  mutate(range = max_date - min_date)
)

#Examine data spread by temp logger ID
boxplot(avg ~ logger_id, data = temp, xlab = "Logger ID", 
        ylab = "Daily average temperature (°C)", las = 2)
boxplot(sd ~ logger_id, data = temp, xlab = "Logger ID", 
        ylab = "Daily average temperature (°C)", las = 2)
boxplot(max ~ logger_id, data = temp, xlab = "Logger ID", 
        ylab = "Daily average temperature (°C)", las = 2)
#Logger id is an important source of random variation


#2.1 PLOT DATA IN TIMESERIES

#Average data across loggers for elevations on each date
temp1 <- temp %>% 
  group_by(cal_date, elevation) %>% 
  summarize(avg = mean(avg), min = min(min), max = max(max))

#Save colour palette for graphing
elev_cols <- c("deepskyblue","gold","darkorange","firebrick2")

#Plot timeseries with mean daily temp (line) and range (shaded area above/below line)
(ts <- ggplot(temp1, aes(x = cal_date)) +
  facet_wrap(~elevation, ncol = 1, strip.position = "right") +
  geom_line(aes(y = avg, colour = elevation),
            show.legend = F) +
  geom_ribbon(aes(ymin = min, ymax = max, fill = elevation),
              show.legend = F, alpha = 0.2) +
  scale_color_manual(values = elev_cols) +
  scale_fill_manual(values = elev_cols) +
  theme_minimal(base_size = 20) +
  labs(x = NULL, y = "Temperature (°C)") +
  theme(axis.title.y = element_text(vjust = 3.5)) +
  scale_x_date(date_labels = "%d %B %Y", date_breaks = "2 weeks")
)


#2.2 MODEL DATA USING GAMMs

#Is response variable normal?
hist(temp$avg) #Pretty good

#Random effects selection
#Base GAMM without random intercept for temp logger id, random smooth for date
avg.m0 <- gam(avg ~ elevation + s(day),
          method = "REML",
          data=temp) # Error doesn't appear when mgcv is the only package loaded. 

#Add random effect for logger id and random smooth for elevation
avg.m1 <- gam(avg ~ elevation + s(day) + s(logger_id, bs = 're'),
          method = "REML",
          data=temp)

#Allow each elevation its own random smooth
avg.m2 <- gam(avg ~ elevation + s(day, by=elevation) + s(logger_id, bs = 're'),
          method = "REML",
          data=temp)

AICtab(avg.m0, avg.m1, avg.m2) #avg.m2 nicest fit

#Diagnostic plots
par(mfrow = c(2,2))
gam.check(avg.m2) #Response vs. fitted values and distribution of residuals look good
#Are residual values clustered by date?
plot(x = fitted(avg.m2), y = resid(avg.m2, type = "pearson"), 
     col = temp$date,
     xlab = "Fitted values",
     ylab = "Residuals", cex = 1) #Somewhat.

#Better fit when we incorporate temporal auto-correlation term?
avg.m3 <- gamm(avg ~ elevation + s(day, by=elevation),
           random = list(logger_id=~1),
           data=temp,
           correlation = corAR1(form=~day|logger_id))
#Note: cannot directly compare gamm model to gam using conventional AIC.
#See https://stat.ethz.ch/R-manual/R-devel/library/mgcv/html/mgcv-FAQ.html

#Compare diagnostic plots instead
plot(x = fitted(avg.m3$gam), y = resid(avg.m3$gam, type = "pearson"), 
     col = temp$date,
     xlab = "Fitted values",
     ylab = "Residuals", cex = 1) #More homogeneous error with date
gam.check(avg.m3$gam) #More normal residual distribution

#Inferences about elevation differences
#Successive differences contrast matrix
contr <- matrix(0, nrow = 3, ncol = length(coef(avg.m3$gam)))
colnames(contr) <- names(coef(avg.m3$gam))
rownames(contr) <- c("High - Middle", "Middle - Low", "Low - Subtidal")
contr[, 1:4] <- rbind(c(0, 0, -1, 1), c(0, -1, 1, 0), c(-1, 1, 0, 0))
contr[, 1:4]

#Successive differences contrast test
summary(glht(avg.m3$gam, linfct = contr))


#2.3 MODEL DAILY STANDARD DEVIATIONS

#Check normality of temperature standard deviations
hist(temp$sd) #Left-skewed
hist(log(temp$sd +1)) #Normal-er (probably the best we can do) 

#Base GAMM without random intercept for temp logger id, with random smooth for date
sd.m0 <- gam(log(sd + 1) ~ elevation + s(day),
          method = "REML",
          data=temp) # Error doesn't appear when mgcv is the only package loaded. 

#Add random effect for logger id and random smooth for elevation
sd.m1 <- gam(log(sd + 1) ~ elevation + s(day) + s(logger_id, bs = 're'),
          method = "REML",
          data=temp)

#Allow each elevation its own random smooth
sd.m2 <- gam(log(sd + 1) ~ elevation + s(day, by=elevation) + s(logger_id, bs = 're'),
          method = "REML",
          data=temp)

AICtab(sd.m0, sd.m1, sd.m2) #sd.m2 nicest fit

#Check fit
gam.check(sd.m2) #Residual distributions and fitted vs. response not bad... increase k?

#Add two knots (from default 10 to 12)
sd.m3 <- gam(log(sd + 1) ~ elevation + s(day, by=elevation, k = 12) + 
               s(logger_id, bs = 're'),
          method = "REML",
          data=temp)

#Check fit
gam.check(sd.m3) #Increasing k seems to have helped. More?

#Double default knots to 20
sd.m4 <- gam(log(sd + 1) ~ elevation + s(day, by=elevation, k = 20) + 
               s(logger_id, bs = 're'),
             method = "REML",
             data=temp)

gam.check(sd.m4) #This is a nicer fit.

plot(x = fitted(sd.m4), y = resid(sd.m4, type = "pearson"), 
     col = temp$date,
     xlab = "Fitted values",
     ylab = "Residuals", cex = 1) #Temporal correlation likley present

#Standard deviation model with temporal autocorrelation term
sd.m5 <- gamm(log(sd+1) ~ elevation + s(day, by=elevation, k = 20, bs = "cr"),
             random = list(logger_id=~1),
             data=temp,
             correlation = corAR1(form=~day|logger_id))

#Fitting checks
gam.check(sd.m5$gam) #Similar response vs. fitted to sd.m5
plot(x = fitted(sd.m5$gam), y = resid(sd.m5$gam, type = "pearson"), 
     col = temp$date,
     xlab = "Fitted values",
     ylab = "Residuals", cex = 1) #Better

#Contrast matrix for successive differences contrast
contr <- matrix(0, nrow = 3, ncol = length(coef(sd.m5$gam)))
colnames(contr) <- names(coef(sd.m5$gam))
rownames(contr) <- c("High - Middle", "Middle - Low", "Low - Subtidal")
contr[, 1:4] <- rbind(c(0, 0, -1, 1), c(0, -1, 1, 0), c(-1, 1, 0, 0))
contr[, 1:4]

#Successive differences contrast test
summary(glht(sd.m5$gam, linfct = contr))


#2.4 MODEL DAILY MAXIMUM TEMPERATURES

#Normality check in maximum values
hist(temp$max) #Left-skewed, though less severely than sd
hist(log(temp$max)) #Helps, somewhat
hist(sqrt(temp$max + 0.5)) #slightly better than log
hist(sqrt(temp$max + 3/8)) #slightly better than + 0.5 (above)

#Base GAMM without random intercept for temp logger id, with random smooth for date
max.m0 <- gam(sqrt(max + 3/8) ~ elevation + s(day),
             method = "REML",
             data=temp) # Error doesn't appear when mgcv is the only package loaded. 

#Add random effect for logger id and random smooth for elevation
max.m1 <- gam(sqrt(max + 3/8)  ~ elevation + s(day) + s(logger_id, bs = 're'),
             method = "REML",
             data=temp)

#Allow each elevation its own random smooth
max.m2 <- gam(sqrt(max + 3/8)  ~ elevation + s(day, by=elevation) + 
                s(logger_id, bs = 're'),
             method = "REML",
             data=temp)

AICtab(max.m0, max.m1, max.m2) #max.m2 nicest fit

gam.check(max.m2) #response vs. fitted fanning out, try increasing k?

#Increase knots from 10 to 12 and to 20
max.m3 <- gam(sqrt(max + 3/8)  ~ elevation + s(day, by=elevation, k = 12) + 
                s(logger_id, bs = 're'),
              method = "REML",
              data=temp)
max.m4 <- gam(sqrt(max + 3/8)  ~ elevation + s(day, by=elevation, k = 20) + 
                s(logger_id, bs = 're'),
              method = "REML",
              data=temp)
gam.check(max.m3)
gam.check(max.m4) #Tighter response vs. fitted


#Max values model with autocorrelation
max.m5 <- gamm(sqrt(max + 3/8) ~ elevation + s(day, by=elevation, k = 20, bs = "cr"),
              random = list(logger_id=~1),
              data=temp,
              correlation = corAR1(form=~day|logger_id))

gam.check(max.m5$gam) #Tightens residuals a little more


#Contrast matrix
contr <- matrix(0, nrow = 3, ncol = length(coef(max.m5$gam)))
colnames(contr) <- names(coef(max.m5$gam))
rownames(contr) <- c("High - Middle", "Middle - Low", "Low - Subtidal")
contr[, 1:4] <- rbind(c(0, 0, -1, 1), c(0, -1, 1, 0), c(-1, 1, 0, 0))
contr[, 1:4]

#Successive differences contrast test
summary(glht(max.m5$gam, linfct = contr))


#2.5 PLOT FITTED GAMM MODELS

#Make new df for predictions
pframe <- with(temp,expand.grid(
  day=seq(min(day),max(day),length=73),
  elevation=levels(elevation),
  logger_id=NA))

#Shorten date ranges for each elevation to their actual range
pframe$date = rep(134:206, 4)
t1 <- subset(pframe, elevation == "High" & date > 136 & date < 192)
t2 <- subset(pframe, elevation == "Middle")
t3 <- subset(pframe, elevation == "Low" & date > 137)
t4 <- subset(pframe, elevation == "Subtidal" & date > 139)
pframe <- rbind(t1,t2,t3,t4)


#Plot of model predictions: averages
pframe$pred.avg <- predict.gam(avg.m3$gam,newdata = pframe,type="response", re.form=NA)
pframe$SE.avg <- predict.gam(avg.m3$gam,newdata = pframe,type="response", 
                         re.form=NA, se.fit = TRUE)$se.fit

z.val <- qnorm(1 - (1 - 0.9)/2)

pframe$LoCI.avg <- pframe$pred.avg - (z.val * pframe$SE.avg)
pframe$HiCI.avg <- pframe$pred.avg + (z.val * pframe$SE.avg)

#Plot averages
ggplot(pframe, aes(as.Date(pframe$date, origin = "1900-01-01"), 
                   pred.avg, colour=elevation)) +
  geom_point(data = temp, aes(x = as.Date(temp$date, origin = "1900-01-01"), 
                              y = avg),
             cex = 1.5, alpha = 0.5) +
  geom_ribbon(aes(ymin = LoCI.avg, ymax = HiCI.avg, group = elevation), 
              alpha = 0.15, colour = NA, show.legend = FALSE) +
  geom_line(size = 1.3, show.legend = FALSE) +
  scale_color_manual(breaks=c("Subtidal","Low", "Middle","High"),
                     values=c("deepskyblue","gold","darkorange","firebrick2")) +
  labs(x=NULL, y="Daily average temperature (°C)", colour = "Intertidal elevation") +
  scale_x_date(labels = date_format("%d %B"),breaks = "2 weeks") +
  scale_y_continuous(breaks = c(14, 16, 18, 20)) +
  coord_cartesian(ylim = c(13, 21)) +
  theme_classic(base_size = 26) +
  theme(axis.title.y = element_text(vjust = 3.5),
        legend.position = c(0.4, 0.8))

#Plot of model predictions: sd
pframe$pred.sd <- predict.gam(sd.m5$gam,newdata=pframe,type="response", re.form=NA)
pframe$SE.sd <- predict.gam(sd.m5$gam,newdata=pframe,type="response",
                            re.form=NA, se.fit = TRUE)$se.fit

#Inverse transform predictinos and calculate 90/10 CIs
pframe$LoCI.sd <- exp(pframe$pred.sd - (z.val * pframe$SE.sd)) - 1
pframe$HiCI.sd <- exp(pframe$pred.sd + (z.val * pframe$SE.sd)) - 1
pframe$pred.sd <- exp(pframe$pred.sd) - 1

#Plot
ggplot(pframe, aes(as.Date(pframe$date, origin = "1900-01-01"), 
                   pred.sd, colour=elevation)) +
  geom_point(data = temp, aes(x = as.Date(temp$date, origin = "1900-01-01"), y = sd),
             cex = 1.5, alpha = 0.5
             , show.legend = F) +
  geom_ribbon(aes(ymin = LoCI.sd, ymax = HiCI.sd, group = elevation), 
              alpha = 0.15, colour = NA, show.legend = FALSE) +
  geom_line(size = 1.3, show.legend = FALSE) +
  scale_color_manual(breaks=c("Subtidal","Low", "Middle","High"),
                     values=c("deepskyblue","gold","darkorange","firebrick2")) +
  labs(x=NULL, y="Daily temperature standard\ndeviation (°C)",
       colour = "Intertidal elevation") +
  scale_x_date(labels = date_format("%d %B"),breaks = "2 weeks") +
  #scale_y_continuous(breaks = c(14, 16, 18, 20)) +
  #coord_cartesian(ylim = c(12, 21)) +
  theme_classic(base_size = 26) +
  theme(axis.title.y = element_text(vjust = 3.5)
        #,legend.position = c(0.4, 0.8)
  )


#Plot of model predictions: max
pframe$pred.max <- predict.gam(max.m5$gam,newdata=pframe,type="response", re.form=NA)
pframe$SE.max <- predict.gam(max.m5$gam,newdata=pframe,type="response",
                             re.form=NA, se.fit = TRUE)$se.fit

#Inverse transform predictinos and calculate 90/10 CIs
pframe$LoCI.max <- (pframe$pred.max - (z.val * pframe$SE.max))^2 - (3/8)
pframe$HiCI.max <- (pframe$pred.max + (z.val * pframe$SE.max))^2 - (3/8)
pframe$pred.max <- (pframe$pred.max)^2 - 3/8

#Plot
ggplot(pframe, aes(as.Date(pframe$date, origin = "1900-01-01"), 
                   pred.max, colour=elevation)) +
  geom_point(data = temp, aes(x = as.Date(temp$date, origin = "1900-01-01"), y = max),
             cex = 1.5, alpha = 0.5
             , show.legend = F
  ) +
  geom_ribbon(aes(ymin = LoCI.max, ymax = HiCI.max, group = elevation), 
              alpha = 0.15, colour = NA, show.legend = FALSE) +
  geom_line(size = 1.3, show.legend = FALSE) +
  scale_color_manual(breaks=c("Subtidal","Low", "Middle","High"),
                     values=c("deepskyblue","gold","darkorange","firebrick2")) +
  labs(x=NULL, y="Daily maximum temperature (°C)",
       colour = "Intertidal elevation") +
  scale_x_date(labels = date_format("%d %B"),breaks = "2 weeks") +
  #scale_y_continuous(breaks = c(14, 16, 18, 20)) +
  #coord_cartesian(ylim = c(12, 21)) +
  theme_classic(base_size = 26) +
  theme(axis.title.y = element_text(vjust = 3.5)
        #,legend.position = c(0.4, 0.8)
  )
# 3) Nest uptake and spawning data -------------------------------------------

#3.1 MALE BODY SIZES AT UPTAKE

#Load body size data
bs.data <- read.csv("elevation_bodycondition_v1.csv") %>% 
  mutate(date = Date %>% as.character() %>% as.Date(format = "%d-%b-%y") %>% 
           format("%j") %>% as.numeric(),
         elevation = factor(elevation, levels = c("l", "m", "h"),
                            labels = c("Low", "Middle", "High"))) %>%
  filter(mass_GM > 50) #Remove fish likely to be sneakers or females

#Summary stats for standard lengths
bs.data %>%
  group_by(elevation) %>% 
  dplyr::summarize(mean_SL = mean(SL_GM), sd_SL = sd(SL_GM), 
                   min_SL = min(SL_GM), max_SL = max(SL_GM), N = dplyr::n())

#Fig 2.2A
ggplot(bs.data, aes(x=elevation, y=SL_GM))+
  geom_boxplot(fatten = 3, outlier.shape = NA, show.legend = FALSE,
               aes(colour = elevation, fill = elevation), alpha = 0.2)+
  geom_jitter(position=position_jitter(0.2), cex = 2) +
  labs(x = NULL, y = "Male standard length (mm)") +
  scale_colour_manual(values = c("gold", "darkorange", "firebrick2"),
                      aesthetics = c("colour", "fill"))+
  scale_y_continuous(breaks=seq(0,320,50))+
  theme_classic(base_size = 22) +
  theme(axis.title.y = element_text(vjust = 3.5))

#Specify custom contrast for standard length
mvlh <- c(-0.5, 1, -0.5)
contrasts(bs.data$elevation) <- cbind(mvlh)

#Contrast test
summary(lm(SL_GM ~ elevation, data = bs.data)) #Middle fish not the largest
TukeyHSD(aov(SL_GM ~ elevation, data = bs.data)) #Low fish are larger
anova(aov(SL_GM ~ elevation, data = bs.data))


#3.2 NEST UPTAKE PROPORTIONS

#Input data and build table
spawn <- read.csv('spawn.csv') %>% 
  mutate(elevation = factor(elevation, levels = c('Subtidal', 'Low', 'Middle', 'High')))

#binomial GLM
sp <- glm(cbind(clutch, fail) ~ elevation, family = binomial(), data = spawn)
summary(sp)

#Inference
Anova(sp)

#Store model predictions
spdat <- data.frame(elevation = levels(spawn$elevation)) %>% 
  mutate(elevation = factor(elevation, levels = c('Subtidal', 'Low', 'Middle', 'High')))

#Get binomial CI's
## grad the inverse link function 
ilink <- family(sp)$linkinv 
## add fit and se.fit on the link scale 
spdat <- bind_cols(spdat, setNames(as_tibble(predict(sp, spdat, se.fit = TRUE)[1:2]), 
                                   c('fit_link','se_link'))) 
## create the interval and backtransform 
spdat <- mutate(spdat, fit_resp = ilink(fit_link), 
                upr = ilink(fit_link + (qnorm(1 - (1 - 0.9)/2) * se_link)), 
                lwr = ilink(fit_link - (qnorm(1 - (1 - 0.9)/2) * se_link)))

#Fig 2.2B
ggplot(spdat, aes(x = elevation, y = fit_resp))+
  geom_bar(stat = "identity", aes(fill = elevation), colour="black", 
           show.legend = FALSE) +
  geom_errorbar(aes(ymin = lwr, ymax = upr), width = 0.25) +
  scale_fill_manual(values=c("deepskyblue", "gold", "darkorange","firebrick2")) +
  labs(x = NULL, y = "Proportion of nests with eggs") +
  scale_y_continuous(breaks=seq(0,1,0.25))+
  theme_classic(base_size = 22) +
  theme(axis.title.y = element_text(vjust = 3.5))


#3.3 NUMBER OF EGGS ACQUIRED

#Load all egg data
eggs.all <- read.csv("eggs_by_elevation.csv") %>% 
  mutate(male_sl = male_sl *10, date = date %>% as.character() %>% 
           as.Date(format = "%d-%b") %>% format("%j") %>% as.numeric(),
         elevation = factor(elevation_num, labels = c("Subtidal", "Low", 
                                                      "Middle", "High")),
         manip = factor(manip, labels = c("Experimental nests", 
                                          "Natural nests")))

#First measurements for experimental and natural nests
eggs.b <- eggs.all %>%
  filter(!duplicated(nest_id)) %>% #Remove all but first observation
  mutate(eggs_pres = if_else(all_eggs == 0, 0, 1))

#Summary data
eggs.b %>% 
  group_by(manip, elevation) %>% 
  summarise(median = median(all_eggs), q25 = quantile(all_eggs, probs = 0.25),
            q75 = quantile(all_eggs, probs = 0.75))

#colour palette for graphs
elev_cols <- c("deepskyblue", "gold", "darkorange", "firebrick2")

#Fig. 2.2C
ggplot(eggs.all %>% filter(!elevation == "Subtidal"), 
       aes(elevation, all_eggs)) +
  facet_wrap(~manip) +
  geom_boxplot(aes(colour = elevation, fill = elevation), 
               fatten = 3, alpha = 0.2, outlier.alpha = 0,
               show.legend = F) +
  geom_jitter(position = position_jitter(0.3), aes(size = male_sl),
              alpha = 0.5) +
  scale_color_manual(values = c("goldenrod1", "darkorange", "firebrick2"),
                     aesthetics = c("colour", "fill")) +
  labs(x = NULL, y = "Number of eggs in nest", size = "Male\nstandard\nlength (mm)") +
  theme_classic(base_size = 22) +
  theme(axis.title.y = element_text(vjust=3.5),
        legend.position = c(0.15, 0.8))

#Distribution of data
hist(eggs.all[!is.na(eggs.all$male_sl),]$all_eggs) #Negative binomial or similar

#Male experimental and natural nests with male SL data
#sl1 <- glmmTMB(all_eggs ~ male_sl + elevation + manip + (1|nest_id),
#               eggs.all,
#               family = compois()) #long run time
sl2 <- glmmTMB(all_eggs ~ male_sl + elevation + manip + (1|nest_id),
               eggs.all,
               family = genpois())
sl3 <- glmmTMB(all_eggs ~ male_sl + elevation + manip + (1|nest_id),
               eggs.all,
               family = poisson())
sl4 <- glmmTMB(all_eggs ~ male_sl + elevation + manip + (1|nest_id),
               eggs.all,
               family = nbinom1())
sl5 <- glmmTMB(all_eggs ~ male_sl + elevation + manip + (1|nest_id),
               eggs.all,
               family = nbinom2())
AICctab(sl2,sl3,sl4,sl5) #Sl5 offers the best fit

#Try with ZI parameter
#sl6 <- glmmTMB(all_eggs ~ male_sl + elevation + manip,
#               eggs.b, ziformula = ~1,
#               family = truncated_compois()) #long run time
sl7 <- glmmTMB(all_eggs ~ male_sl + elevation + manip,
               eggs.b, ziformula = ~1,
               family = truncated_nbinom2()) #convergence failure

#Inference
car::Anova(sl5)
exp(coef(summary(sl5))$cond[2,-c(3,4)]) #Slope of male SL effect

#Difference between experimental and natural nests
emmeans(sl5, pairwise ~ manip, transform = "response")
#Natural nests get 938 more eggs, on average


# 4) Costs of care analyses --------------------------------------------------

#4.1 BODY CONDITION DETERIORATION MODEL

#Load and prepare data
bc.change <- read.csv('RM_body_condition.csv') %>% 
  mutate(fultons_k = (mass)*10^5/(sl*10)^3,
         cond = resid(lmer(log(mass)~log(10*sl) + (1|male_id))),
         date = date %>% as.character() %>% as.Date(format = "%d-%b-%y") %>% 
           format("%j") %>% as.numeric(),
         elevation = factor(elevation, levels = c("l", "m", "h"),
                            labels = c("Low", "Middle", "High")))

#Do two body condition measures correlate?
summary(lm(cond ~ fultons_k, bc.change)) #Very much so.

#Plot individual timeseries of condition index
bc_cols <- c("gold", "darkorange", "firebrick2")
bcp0 <- ggplot(bc.change) +  
  geom_point(aes(x = age, y = cond, color = elevation), size = 3) +
  labs(x = "Elapsed days", y = "Condition index", colour="Intertidal\nelevation") +
  scale_colour_manual(values=bc_cols) +
  theme_classic(base_size = 22) +
  theme(axis.title.y = element_text(vjust = 2),
        legend.position = c(0.75, 0.8))

bcp0 +   
  geom_path(aes(x = age, y = cond, group = interaction(elevation, male_id),
                color = elevation))
  
#Build candidate models for condition

bc0 <- gls(cond ~ elevation*scale(age, scale = FALSE), 
           method = "REML", data = bc.change)
bc1 <- lmer(cond ~ elevation*scale(age, scale = FALSE) + (1|male_id), 
            REML = T, data = bc.change)
bc2 <- lmer(cond ~ elevation*scale(age, scale = FALSE) + 
              (1+scale(age, scale = FALSE)|male_id), 
            REML = T,data = bc.change) #Convergence warning
bc3 <- update(bc2, .~., control = lmerControl(optimizer = "Nelder_Mead")) 
#Resolves issue

AICctab(bc0, bc1, bc3) #bc3 preferred

#Fit hypothesis test with ML
H1 <- lmer(cond ~ elevation*scale(age, scale = FALSE) + 
             (1+scale(age, scale = FALSE)|male_id), 
           contrasts = list(elevation = 'contr.sum'), #set intercept to grand mean
           REML = FALSE, data = bc.change,
           control = lmerControl(optimizer = "Nelder_Mead"))

#Check residuals 
plot(H1, id = 0.01) #Outlier in low elevation. No biological reason to exclude.
plot(H1,elevation~resid(.,type="pearson")) #by group

#Successive differences contrast: test of main hypothesis
skip_comp.emmc(1:3, skip = 0, reverse = TRUE) #Contrast matrix: L < M, M < H
contrast(emtrends(H1, ~elevation, var="age"), 
         "skip_comp", skip = 0, reverse = TRUE)

#Average condition of males at each elevation
emmeans(H1, ~elevation)
#differences
contrast(emmeans(H1, ~elevation), 
         "skip_comp", skip = 0, reverse = TRUE)

#inference
Anova(H1, type = 3) #ANOVA

#Examine magnitude of differences between groups
#Fit model without scaled age to get interpretable coefficients
H1_unsc <- lmer(cond ~ elevation*age + (1+age|male_id),
                REML = FALSE, data = bc.change, 
                contrasts = list(elevation = 'contr.sum'), #set intercept to grand mean
                control = lmerControl(optimizer = "Nelder_Mead"))

#How much did bc change per day on average?
coef(summary(H1_unsc))[4,-3]/fixef(H1_unsc)[1] *100

#Calculate percentage loss of  body condition per day for each elevation
summary(emtrends(H1_unsc,~ elevation, var = "age"))[,2:3] / fixef(H1_unsc)[1] * 100


#4.2 PLOT BODY CONDITION DETERIORATION MODEL PREDICTIONS

#Make newdf for predictions
bc.pred <- with(bc.change,expand.grid(
  age=seq.int(min(age),max(age),length=85),
  elevation=levels(elevation),
  male_id=NA))

#Must de-center and de-scale covariate (age) for accurate predictions
#From https://gist.github.com/ashenkin/8b3a32ced4ee599ed1d76f65a8d3c192
age_center = attributes(H1@frame$"scale(age, scale = FALSE)")[["scaled:center"]]
bc.pred$age_c <- scale(bc.pred$age, center = age_center, scale = FALSE)

#APredict values from model
bc.pred$y <- predict(H1, bc.pred, re.form = NA)

#Extract model matrix for confidence interval calculations
mm <- model.matrix(~ elevation*age_c, bc.pred)

# Variance-covariance matrix of H1 for calculations
vcov.m <- vcov(H1)

# Calculate 90/10 CI of predictions
vars <- mm %*% vcov.m %*% t(mm)
sds <- sqrt(diag(vars))
z.val <- qnorm(1 - (1 - 0.9)/2)
bc.pred$LoCI <- bc.pred$y - z.val * sds
bc.pred$HiCI <- bc.pred$y + z.val * sds

#Plot model predictions and confidence interval: Fig. 2.4A
bcp0 + 
  geom_smooth(data = bc.pred, aes(x = age, y = y, ymin = LoCI, ymax = HiCI,
                                  colour = elevation),
              stat = "identity", alpha = 0.15)
  

#4.3 TIME ON THE NEST ANALYSIS

#Time on the nest
t_nest <- bc.change %>% filter(!int == 0) %>% group_by(male_id) %>%
  mutate(init_cond = max(cond)) %>%  top_n(1, age)

#Group summaries
t_nest %>% group_by(elevation) %>% 
  summarise(median_age = median(age), q25 = quantile(age, probs = 0.25),
            q75 = quantile(age, probs = 0.75), IQR = IQR(age), N = n())

#Visualize time on the nest
ggplot(t_nest, aes(elevation, age)) +
  geom_boxplot(fatten = 3, aes(colour = elevation, fill = elevation),
               alpha = 0.25, outlier.alpha = 0, show.legend = F) +
  geom_jitter(cex = 2, aes(colour = elevation), 
              position = position_jitter(0.3, seed = 2),
              show.legend = F) +
  labs(x = NULL, y = "Time on the nest (days)") +
  scale_colour_manual(values = bc_cols,
                      aesthetics = c("colour", "fill")) +
  theme_classic(base_size = 22) +
  theme(axis.title.y = element_text(vjust = 3.5),
        legend.position = c(0.8, 0.8))

#Time on nest ANOVA and successive differences contrast
t <- aov(sqrt(age+3/8) ~ elevation + init_cond, t_nest)
summary(t)
contrast(emmeans(t, pairwise~elevation), 
         "skip_comp", skip = 0, reverse = TRUE)
# 5) Parental care data ---------------------------------------------------

#Load/prepare data
pnpc <- read.csv("NY_Dataset_Final_Full-NB.csv") %>% 
  mutate(date = date %>% as.character() %>% as.Date(format = "%d-%m-%y") %>% 
           format("%j") %>% as.numeric(),
         elevation = factor(elevation, levels = c("low", "mid", "high"),
                            labels = c("Low", "Middle", "High")),
         pc = eggdir_f + nestmain_f, #parental care (pc): sum related behaviours
         sl = male_sl*10) %>%
  filter(!is.na(reposition) & !n_eggs == 0)

#Visualize PC count data
pc_cols = c("gold", "darkorange", "firebrick2")
ggplot(pnpc, aes(x=pc)) +
  facet_grid(elevation ~ .) +
  geom_histogram(aes(y=..density.., fill = elevation),
                 binwidth=5, colour="black", show.legend = F) +
  geom_density(alpha=.4, aes(fill = elevation), show.legend = F) +
  scale_color_manual(values = pc_cols,
                     aesthetics = c("colour", "fill")) +
  labs(x = "Parental Care Frequency", y = "Density") +
  theme_classic(base_size = 22) +
  theme(axis.title.y = element_text(vjust = 3.5))

#Boxplot: Figure 2.4B
ggplot(pnpc, aes(elevation, pc)) +
  geom_boxplot(aes(colour = elevation, fill = elevation), fatten = 3,
               show.legend = FALSE, alpha = 0.2, outlier.shape = NA) +
  geom_jitter(position = position_jitter(0.2, seed = 7), cex = 3
              #,aes(size = sl) #Can include male sl (or n_eggs)
              ) +
  labs(x = NULL, y = "Number of care behaviours\nexhibited in 60 minutes") +
  scale_colour_manual(values = c("gold", "darkorange", "firebrick2"),
                      aesthetics = c("colour", "fill")) +
  scale_y_continuous(breaks=seq(0,320,50))+
  theme(axis.title.y = element_text(vjust = 3.5)) +
  theme_classic(base_size = 22)

#Summarize data
pnpc %>% 
  group_by(elevation) %>% 
  summarise(N = n(), n_males = length(unique(maleid)), median = median(pc),
            IQR = IQR(pc))

#Select best error distribution to model count data
pcf.nb0 <- glmmTMB(pc ~ elevation * (scale(sl) + scale(n_eggs)) + (1|maleid),
                   family=nbinom1, data=pnpc)
pcf.nb1 <- glmmTMB(pc ~ elevation * (scale(sl) + scale(n_eggs)) + (1|maleid),
                   family=nbinom2, data=pnpc)
pcf.p0 <- glmmTMB(pc ~ elevation * (scale(sl) + scale(n_eggs)) + (1|maleid),
                  family=poisson(), data=pnpc)
pcf.p1 <- glmmTMB(pc ~ elevation * (scale(sl) + scale(n_eggs)) + (1|maleid),
                  family=genpois(), data=pnpc)
#pcf.p2 <- glmmTMB(pc ~ elevation * (scale(sl) + scale(n_eggs)) + (1|maleid),
#                  family=compois(), data=pnpc)
pcf0 <- glmmTMB(pc ~ elevation * (scale(sl) + scale(n_eggs)) + (1|maleid),
                ziformula=~1,
                family=nbinom1, data=pnpc)
pcf1 <- glmmTMB(pc ~ elevation * (scale(sl) + scale(n_eggs)) + (1|maleid),
                ziformula=~1,
                family=nbinom2, data=pnpc)
pcf2 <- glmmTMB(pc ~ elevation * (scale(sl) + scale(n_eggs)) + (1|maleid),
                ziformula=~(1|maleid),
                family=nbinom1, data=pnpc)
pcf3 <- glmmTMB(pc ~ elevation * (scale(sl) + scale(n_eggs)) + (1|maleid),
                ziformula=~(1|maleid),
                family=nbinom2, data=pnpc)
AICctab(pcf.nb0,pcf.nb1,pcf.p0,pcf.p1,pcf1,pcf0,pcf2,pcf3)
#Standard negative binomial model fits best

#Re-fit model for inference
pc.m <- glmmTMB(pc ~ elevation * (scale(sl) + scale(n_eggs)) + (1|maleid),
                contrasts = list(elevation = 'contr.sum'), #Use sum contrasts
                family=nbinom1, data=pnpc)

#Successive differences contrast
skip_comp.emmc(1:3, skip = 0) #Matrix: H < M, M < L
contrast(emmeans(pc.m, ~ elevation, transform = "response"), 
         "skip_comp", skip = 0, adjust = NULL) #Pair-wise comparisons High < middle < low

#Inference for covariates and interactions
Anova(pc.m, type = 3) #ANOVA: set options(contrasts=c('contr.sum','contr.poly'))

#Calculate number of behaviours predicted from 1 mm increase in male SL
coef(summary(pc.m))$cond[4,c(1,2)] * sd(pnpc$sl, na.rm = T) + mean(pnpc$sl, na.rm = T)


# 6) Brood development data -----------------------------------------------

#6.1 MODEL DEVELOPMENT DATA

#Load/prep data
devo <- (read.csv('development_at_elevation.csv') %>% 
           mutate(date = date %>% as.character() %>% as.Date(format="%d-%b")
                  %>% format("%j") %>% as.numeric(),
                  age_cs=scale(age), #center and scale age
                  elevation=factor(elevation_num,
                                   levels=rev(1:4),
                                   labels=rev(c("High", "Middle", 
                                                "Low", "Subtidal")))) %>%
           filter(date > 133) #Remove early May data
)

#Make second ataset for modelling (drops initial measurements)
devo1 <- devo[!devo$int==0, ] %>% 
  mutate(logint_s = log(scale(int, center=FALSE)))

dev_cols <- c("deepskyblue","gold", "darkorange", "firebrick2")
#Graph development rates
(dp0 <- ggplot(devo,aes(age, prop_devo, colour=elevation))+
    #facet_wrap(~elevation, ncol = 2) +
    geom_jitter(position = position_dodge(4), 
                aes(size=n), alpha = 0.7)+
    labs(colour="Intertidal\nelevation", size="Brood size", 
         x = "Elapsed days", y = "Proportion of total development") +
    theme_classic(base_size = 22) +
    scale_colour_manual(values=dev_cols) +
    theme(axis.title.y = element_text(vjust = 3.5),
          legend.position = c(0.7, 0.2),
          legend.box = "horizontal")
) #Don't mind warning about dodging continuous x values

#Show individual development trajectories
dp0 + geom_path(aes(group = interaction(elevation_num, clutch_id)), cex = 0.3,
                show.legend = FALSE) 

#Candidate models
dev <- glmer(prop_devo ~ elevation*scale(age, scale = FALSE) +
               offset(logint_s) + (1|clutch_id),
             weights=n,
             control = glmerControl(optimizer = "bobyqa"),
             family = binomial(link = "cloglog"), data = devo1)
dev2 <- update(dev, #Move int to fixed effects
               . ~ elevation*scale(age, scale = FALSE) + logint_s + 
                 (1|clutch_id),
               control=glmerControl(optimizer="nloptwrap",optCtrl=
                                      list(xtol_abs=1e-10, ftol_abs=1e-10)))
dev3 <- glmer(prop_devo ~ elevation*scale(age, scale = FALSE) + logint_s + 
                (1|clutch_id),
              weights=n,
              control = glmerControl(optimizer = "bobyqa"),
              family = binomial(link = "logit"), data = devo1) #Try with logit link
AICtab(dev,dev2, dev3)  #log(int) as a *predictor* (not offset) is much better, 
#logit link slightly better than cloglog

#Model check
plot(dev3,elevation~resid(.,type="pearson")) #Higher residual values in subtidal

#Put parameter values in units per day
age_center = attributes(dev3@frame$"scale(age, scale = FALSE)")[["scaled:center"]]
binomial(link="logit")$linkinv(fixef(dev3)) + age_center#Not sure this makes sense...

#Re-fit model with contrasts for inference
dev.m <- glmer(prop_devo ~ elevation*scale(age, scale = FALSE) + logint_s + 
                (1|clutch_id),
              weights=n, contrasts = list(elevation = 'contr.sum'),
              control = glmerControl(optimizer = "bobyqa"),
              family = binomial(link = "logit"), data = devo1)

#Inference
Anova(dev.m, type = 3) #ANOVA

#Developmental rate differences between elevations
#calculate % difference in development progressed per day
skip_comp.emmc(1:4, reverse = T, skip = 0) #Matrix: H > M, M > L, L > S
summary(contrast(emtrends(dev.m, ~ elevation,var = "age",transform = "response"),
         "skip_comp", reverse = TRUE, skip = 0, adjust = NULL))[,2:3] * 100


#6.2 PLOT MODEL PREDICTIONS

#New data frame for predictions
dev_pred <- with(devo,expand.grid(
  age=seq.int(min(age),max(age),length=151),
  elevation=levels(elevation),
  logint_s=mean(devo1$logint_s),
  clutch_id=NA))

#Age decentring/descaling
age_center = attributes(dev3@frame$"scale(age)")[["scaled:center"]]
dev_pred$age_cs <- scale(dev_pred$age, center = age_center, scale = age_scale)

#Predict
dev_pred$y <- predict(dev3, dev_pred, type = "link", re.form = NA)

#model matrix for 90/10CI calculations
mm <- model.matrix(~ elevation*age_cs+logint_s, dev_pred)
vcov.m <- vcov(dev3)
vars <- mm %*% vcov.m %*% t(mm)
sds <- sqrt(diag(vars))

#Calculate 90/10 CI
z.val <- qnorm(1 - (1 - 0.9)/2)
dev_pred$Lo90CI <- dev_pred$y - z.val * sds
dev_pred$Hi90CI <- dev_pred$y + z.val * sds

#Inverse link function to reverse-transform predictions back to response scale
logit_linkinv <-  binomial(link="logit")$linkinv
#Run predicted values and their confidence bands through inverse link function
dev_pred$y <- logit_linkinv(dev_pred$y)
dev_pred$Lo90CI <- logit_linkinv(dev_pred$Lo90CI)
dev_pred$Hi90CI <- logit_linkinv(dev_pred$Hi90CI)

#Plot Fig. 2.5A
dp0 + geom_smooth(data=dev_pred,aes(x=age,y=y,
                                 ymin=Lo90CI,ymax=Hi90CI),
                  stat="identity",alpha=0.25,show.legend=FALSE)

# 7) Brood mortality data -------------------------------------------------

#7.1 MODEL BROOD MORTALITY
#Note: I recieved lots of help from Ben Bolker in this section (and also section 6)

#Load and prepare data
bs <- (read.csv("proportion_across_season_v3.csv")               
       %>% rename(surv = Proportion.alive,
                  id = Nest.ID,
                  date = Date,
                  male = Male_ID,
                  mass = mass_GM,
                  sl = SL_GM)
       %>% mutate(date = date %>% as.character() %>% as.Date(format="%d-%b-%y")
                  %>% format("%j") %>% as.numeric(),
                  age_cs=scale(age), #center and scale age
                  elevation=factor(elevation, levels=c("l","m","h"),
                                   labels=c("Low","Middle","High"))
       )
)

#Dataset for modelling: drop initial observations
bs1 <- (bs
        %>% filter(int!=0)
        %>% mutate(age_cs=drop(scale(age)), #scale and center age
                   age_s = c(age_cs - min(age_cs)), #set min value for scaled age to 0
                   int_s = c(drop(scale(int, center=FALSE))),
                   logint_s=log(int_s)
        )
)

mort_colours <- c("gold","darkorange","firebrick2")
#Plot survival curves
sp0 <- ggplot(bs,aes(x=age,y=surv,colour=elevation))+
  geom_point(aes(size = live), alpha = 0.75)+
  labs(size="Living eggs", x = "Elapsed days", 
       y = "Proportion of brood alive", colour = "Intertidal\nelevation") +
  coord_cartesian(ylim = c(0,1), xlim = c(0,60)) +
  theme_classic(base_size = 22) +
  scale_colour_manual(values=mort_colours) +
  theme(axis.title.y = element_text(vjust = 3.5),
        legend.position = c(0.8, 0.7))


#Show individual mortality trajectories
sp0 + geom_path(aes(group = interaction(elevation, clutch_id)), cex = 0.3,
                show.legend = FALSE)

#Candidate models
bs.m <- glmer(cbind(died, live) ~ elevation*scale(age, scale = FALSE) + 
                offset(logint_s) + (1|clutch_id), 
              family = binomial(link = "cloglog"),
              data = bs1)

#With logint_s as predictor rather than offset
try(bs.m2 <- update(bs.m,
                    . ~ elevation*scale(age, scale = FALSE) + logint_s + (1|clutch_id),
                    control=glmerControl(optimizer="nloptwrap",optCtrl=
                                           list(xtol_abs=1e-10, ftol_abs=1e-10))))
#Convergence warnings...
try(bs.m2 <- update(bs.m,
                    . ~ elevation*scale(age, scale = FALSE) + logint_s + (1|clutch_id),
                    control=glmerControl(optimizer="bobyqa"))
)

try(bs.m3 <- update(bs.m,
                    . ~ elevation*scale(age, scale = FALSE) + logint_s + (1|clutch_id),
                    control=glmerControl(optimizer="nlminbwrap",optCtrl=
                                           list(step.min = 2, step.max = 2)))
    #increasing step size allows convergence, but what does this mean?...
)
# Convergence warnings

bs.m4 <- glm(cbind(died,live) ~ elevation*scale(age, scale = FALSE) + logint_s,
             family=binomial(link="cloglog"),
             data=bs1)

#Relative goodness of fit
AICctab(bs.m,bs.m2,bs.m3,bs.m4) #Take bs.m3

#Check values of parameters
fixef(bs.m3) #Not outrageous for cloglog scale

#Refit model with contrasts for inference
try(bs.mi <- update(bs.m,
                    . ~ elevation*scale(age, scale = FALSE) + logint_s + (1|clutch_id),
                    contrasts = list(elevation = 'contr.sum'),
                    control=glmerControl(optimizer="nlminbwrap",optCtrl=
                                           list(step.min = 4, step.max = 4)))
    #increasing step size to 4 allows convergence, but what does this mean?...
)

#Successive differences contrast
skip_comp.emmc(1:3, skip = 0, reverse = T) #Matrix: H > M, M > L
(bs.c <- contrast(emtrends(bs.mi, ~ elevation,var = "age"), 
         "skip_comp", skip = 0, reverse = T, adjust = NULL))
#Calculate hazard (likelihood of mortality per day) increase with elevation
(exp(summary(bs.c)[,2:3]) - 1) *100

#Inference
Anova(bs.mi, type = 3) #ANOVA


#7.2 PLOT BROOD SURVIVAL PREDICTION CURVES

#New dataframe for predictions
newd <- with(bs,expand.grid(age=seq.int(min(age),max(age),length=40),
                            elevation=levels(elevation),
                            logint_s=mean(bs1$logint_s)),
             clutch_id=NA)

#Properly de-center/scale age
#Use model with trt contrasts so elevation levels match
age_cen = attributes(bs.m3@frame$"scale(age, scale = FALSE)")[["scaled:center"]]
#age_sc = attributes(bs.m3@frame$"scale(age)")[["scaled:scale"]]
newd$age_cs <- scale(newd$age, center = age_cen, scale = FALSE)

## try with bs.m3
#Predict
newd$y <- predict(bs.m3, newd, type = "link", re.form=NA)

#Model matrix for 90/10 CI calculations
mm <- model.matrix(~ elevation*age_cs + logint_s, newd)
vcov.m3 <- vcov(bs.m3)

vars.m <- mm %*% vcov.m3 %*% t(mm)
sds.m <- sqrt(diag(vars.m))
z.val <- qnorm(1 - (1 - 0.9)/2)

newd$LoCI.m <- newd$y - z.val * sds.m
newd$HiCI.m <- newd$y + z.val * sds.m

cloglog_inv <- binomial(link="cloglog")$linkinv
#Run predicted values and their confidence bands through invebse link function
newd$y <- cloglog_inv(newd$y)
newd$LoCI.m    <- cloglog_inv(newd$LoCI.m)
newd$HiCI.m    <- cloglog_inv(newd$HiCI.m)

#Raw plot: weird estimates reulting from systematic error when
#back-transforming estimates from the cloglog scale
(sp1 <- sp0 + geom_line(data = newd, aes(y = 1-y, x = age)) +
    geom_ribbon(data = newd, aes(y = 1-y, x = age, 
                                 ymin = 1-LoCI.m, ymax = 1-HiCI.m),
                alpha = 0.15, linetype = 0))

#Solution: Get MARGINAL predictions, by hand (thanks to B.Bolker)

# baseline predictions
b0 <- mm %*% fixef(bs.m3)
stopifnot(all.equal(c(b0),newd$y))
# add among-clutch variability
# could do this either on the basis of conditional effects (ranef())
# or theory
b0m <- outer(c(b0),rnorm(10000,mean=0,sd=attr(VarCorr(bs.m3)[[1]],"stddev")),
             FUN="+")
newd$predict.marginal <- rowMeans(cloglog_inv(b0m))
sp1 + geom_line(data=newd,aes(y=1-predict.marginal))

#Marginal CIs?
#Generate every value with a different set of predictobs
randpreds <- MASS::mvrnorm(10000, mu=fixef(bs.m3), Sigma=vcov(bs.m3))
REvals <- rnorm(10000,mean=0,sd=attr(VarCorr(bs.m3)[[1]],"stddev"))
b0m2 <- (mm %*% t(randpreds)) + rep(REvals,each=nrow(mm))
b0m2_resp <- cloglog_inv(b0m2)
newd$predict.marg_lwr <- apply(b0m2_resp,1, quantile, 0.025)
newd$predict.marg_upr <- apply(b0m2_resp,1, quantile, 0.975)

#Plot with CI's that are overly wide...
sp0 + geom_line(data=newd,aes(y=1-predict.marginal)) +
  geom_ribbon(data=newd,
              aes(y=1-predict.marginal,
                  ymin=1-predict.marg_lwr, ymax=1-predict.marg_upr,
                  group=elevation),
              alpha=0.1,colour=NA)

#Solution: For *each* random choice of of parameters 
#choose REvals, then average them:

get_marg_mean <- function(x,n=1000,sd=attr(VarCorr(bs.m3)[[1]],"stddev"),
                          linkinv=cloglog_inv) {
  rmat <- outer(c(x),rnorm(n,mean=0,sd=sd), FUN="+")
  return(rowMeans(linkinv(rmat)))
}

n_pabs <- 1000
rand_pabs <- MASS::mvrnorm(n_pabs, mu=fixef(bs.m3), Sigma=vcov(bs.m3))
rand_preds <- (mm %*% t(rand_pabs))
pb <- txtProgressBar(style=3,max=n_pabs)
for (i in 1:ncol(rand_preds)) {
  setTxtProgressBar(pb,i)
  rand_preds[,i] <- get_marg_mean(rand_preds[,i])
}
newd$predict.marg_lwr3 <- apply(rand_preds,1, quantile, 0.10)
newd$predict.marg_upr3 <- apply(rand_preds,1, quantile, 0.90)

#Plot Figure 2.5B
sp0 + geom_line(data=newd,aes(y=1-predict.marginal)) +
  geom_ribbon(data=newd,
              aes(y=1-predict.marginal,
                  ymin=1-predict.marg_lwr3, ymax=1-predict.marg_upr3,
                  group=elevation),
              alpha=0.1,colour=NA)
